import { test } from '@util/testing'
import VAlert from '@components/VAlert'
import VIcon from '@components/VIcon'

test('VAlert.vue', ({ mount }) => {
  it('should be closed by default', async () => {
    const wrapper = mount(VAlert)

    expect(wrapper.vm.isActive).toBe(false)
    expect(wrapper.html()).toMatchSnapshot()
  })

  it('should have a close icon', async () => {
    const wrapper = mount(VAlert, {
      propsData: { dismissible: true }
    })

    expect(wrapper.html()).toMatchSnapshot()
  })

  it('should render component with transition', async () => {
    const wrapper = mount(VAlert, {
      propsData: { transition: 'foo' }
    })

    wrapper.setProps({ value: true })
    await wrapper.vm.$nextTick()
    expect(wrapper.hasClass('foo-enter')).toBe(true)
  })

  it('should render component with outline prop', async () => {
    const wrapper = mount(VAlert, {
      propsData: { outline: true }
    })

    expect(wrapper.hasClass('alert--outline')).toBe(true)
  })

  it('should be dismissible', async () => {
    const wrapper = mount(VAlert, {
      propsData: {
        value: true,
        dismissible: true
      }
    })

    const icon = wrapper.find('.alert__dismissible')[0]

    const input = jest.fn(value => wrapper.setProps({ value }))
    wrapper.vm.$on('input', input)

    icon.trigger('click')
    expect(input).toBeCalledWith(false)
    expect(wrapper.html()).toMatchSnapshot()
  })

  it('should have a custom icon', async () => {
    const wrapper = mount(VAlert, {
      propsData: {
        value: true,
        icon: 'list'
      }
    })

    const icon = wrapper.find('.alert__icon')[0]

    expect(icon.text()).toBe('list')
  })

  it('should have no icon', () => {
    const wrapper = mount(VAlert)

    expect(wrapper.contains('.icon')).toBe(false)
  })

  it('should display contextual colors by type', async () => {
    const wrapper = mount(VAlert, {
      propsData: {
        type: 'error'
      }
    })

    expect(wrapper.vm.classes.error).toBe(true)

    wrapper.setProps({ 'type': 'success' })
    await wrapper.vm.$nextTick()
    expect(wrapper.vm.classes.success).toBe(true)

    wrapper.setProps({ 'type': 'warning' })
    await wrapper.vm.$nextTick()
    expect(wrapper.vm.classes.warning).toBe(true)

    wrapper.setProps({ 'type': 'info' })
    await wrapper.vm.$nextTick()
    expect(wrapper.vm.classes.info).toBe(true)
  })

  it('should allow overriding color for contextual alert', async () => {
    const wrapper = mount(VAlert, {
      propsData: {
        type: 'error',
        color: 'primary'
      }
    })

    expect(wrapper.vm.classes.primary).toBe(true)
  })

  it('should allow overriding icon for contextual alert', async () => {
    const wrapper = mount(VAlert, {
      propsData: {
        type: 'error',
        icon: 'block'
      }
    })

    const icon = wrapper.find('.alert__icon')[0]

    expect(icon.text()).toBe('block')
  })
})
